#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<stdio.h>
#include	"lcd.h"

#define	MAXSTRING	40

#define	NUM_REQ		4
#define	REQ_FIRST	0
#define	REQ_SECOND	1
#define	REQ_BLINK	2
#define	REQ_RED		3

#define	REQ_BLINK_DELAY	10UL
#define	REQ_RED_DELAY	100UL

#define	GREEN_LED	PD0
#define	RED_LED		PD1
#define	ICP1		PD4
#define	ICP3		PE7

#define	F_CPU_MHZ	16UL
#define	PRESCALE	256UL
#define	OVERFLOW_PULSE	1024UL
#define	US_PULSE_DOUBLE	((double)PRESCALE / (double)F_CPU_MHZ)

volatile	unsigned	long	timer1 = 0;
volatile	unsigned	long	req[NUM_REQ] = {0, 0, 0, 0};
volatile	unsigned	long	first_time=0, sec_time=0;

ISR(TIMER1_OVF_vect)
{	timer1++;
	
	if(timer1 >= req[REQ_BLINK])	req[REQ_BLINK] = 0;
	if(timer1 >= req[REQ_RED])	req[REQ_RED] = 0;
}

ISR(TIMER1_CAPT_vect)
{	req[REQ_FIRST] = 0;
	first_time = timer1<<10 | ICR1;
}

ISR(TIMER3_CAPT_vect)
{	req[REQ_SECOND] = 0;
	sec_time = timer1<<10 | ICR3;
}

enum state_t { MSG, READY_WAIT, FIRST_RDY, SECOND_RDY};

int	main(void)
{
	enum state_t state = MSG;
	char lcd_string[2][MAXSTRING];

	DDRD = 1<<GREEN_LED | 1<<RED_LED;
	
	DDRD &= ~(1<<ICP1);	// Է ĸó1  Է 
	PORTD |= 1<<ICP1;	// ο Ǯ 

	DDRE &= ~(1<<ICP3);	// Է ĸó3  Է 
	PORTE |= 1<<ICP3;	//  Ǯ
	
	TCCR1B |= 1<<WGM12;	// 10 Ʈ  PWM ߻ 
	TCCR1A |= 1<<WGM11 | 1<<WGM10;
	TCCR1B |= 1<<CS12;	// 256 
	TIMSK |= 1<<TICIE1;	// Է ĸó1 ͷƮ Ȱȭ
	TCCR1B |= 1<<ICES1;	// ¿ 
	
	TCCR3B |= 1<<WGM32;	// 10Ʈ  PWM ߻ 
	TCCR3A |= 1<<WGM31 | 1<<WGM30;
	TCCR3B |= 1<<CS32;	// 256 
	ETIMSK |= 1<<TICIE3;	// Է ĸó3 ͷƮ Ȱȭ
	TCCR3B |= 1<<ICES3;	// ¿ 

	TIMSK |= 1<<TOIE1;	// Ÿ̸1 ÷ ͷƮ Ȱȭ

	TCNT3 = TCNT1 = 0;	// Ÿ̸ ʱȭ
	
	sei();
	LCD_init();

	while(1){
		switch(state){
		case MSG:
			sprintf(lcd_string[0], "%-8ld%-8ld", first_time, sec_time);
			sprintf(lcd_string[1], "dt:%-9ld[us]", \
				(sec_time-first_time)*(unsigned long)US_PULSE_DOUBLE);
			LCD_str_write(0, 0, lcd_string[0]);
			LCD_str_write(1, 0, lcd_string[1]);
						
			PORTD |= 1<<RED_LED;
			req[REQ_RED] = timer1 + REQ_RED_DELAY;
			
			state = READY_WAIT;
			break;
			
		case READY_WAIT:
			if( !req[REQ_RED] ){
				PORTD &= ~(1<<RED_LED);
				req[REQ_FIRST] = req[REQ_SECOND] = 1;
				state = FIRST_RDY;
			}
			break;
			
		case FIRST_RDY:
			if( !req[REQ_FIRST] ){
				PORTD |= 1<<GREEN_LED;
				sprintf(lcd_string[0], "%-8ld", first_time);
				LCD_str_write(0, 0, lcd_string[0]);
				state = SECOND_RDY;
			}else if( !req[REQ_BLINK] ){
				PORTD ^= 1<<GREEN_LED;
				req[REQ_BLINK] = timer1 + REQ_BLINK_DELAY;
			}
			break;
			
		case SECOND_RDY:
			if( !req[REQ_SECOND] ){
				PORTD |= 1<<RED_LED;
				PORTD &= ~(1<<GREEN_LED);

				sprintf(lcd_string[0], "%-8ld", sec_time);
				LCD_str_write(0, 8, lcd_string[0]);

				sprintf(lcd_string[1], \
			 "dt:%-9ld[us]", (sec_time-first_time)*(unsigned long)US_PULSE_DOUBLE);
				LCD_str_write(1, 0, lcd_string[1]);
				
				req[REQ_RED] = timer1 + REQ_RED_DELAY;

				state = READY_WAIT;
			}if( !req[REQ_BLINK] ){
				PORTD ^= 1<<RED_LED;
				req[REQ_BLINK] = timer1 + REQ_BLINK_DELAY;
			}
			break;
		}
	}
	return 0;
}
